﻿using LawFirmBusinessLogic.BusinessLogic;
using LawFirmContracts.BindingModels;
using LawFirmContracts.BusinessLogicsContracts;
using LawFirmContracts.DI;
using LawFirmDataModels.Enums;
using DocumentFormat.OpenXml.Spreadsheet;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace LawFirmView
{
    public partial class FormMain : Form
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _orderLogic;
        private readonly IReportLogic _reportLogic;
        private readonly IWorkProcess _workProcess;
		private readonly IBackUpLogic _backUpLogic;
		public FormMain(ILogger<FormMain> logger, IOrderLogic orderLogic, IReportLogic reportLogic, IWorkProcess workProcess, IBackUpLogic backUpLogic)
        {
            InitializeComponent();
            _logger = logger;
            _orderLogic = orderLogic;
            _reportLogic = reportLogic;
            _workProcess = workProcess;
			_backUpLogic = backUpLogic;
		}

        private void FormMain_Load(object sender, EventArgs e)
        {
            LoadData();
        }
        private void LoadData()
        {
            _logger.LogInformation("Загрузка заказов");
            try
            {
				dataGridView.FillAndConfigGrid(_orderLogic.ReadList(null));
				_logger.LogInformation("Загрузка заказов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки заказов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void компонентыToolStripMenuItem_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormComponents>();
                form.ShowDialog();

        }

        private void пакетыДокументовToolStripMenuItem_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormDocuments>();
			form.ShowDialog();
            
        }

        private void buttonCreateOrder_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormCreateOrder>();
			form.ShowDialog();
                LoadData();
            

        }

        private void buttonIssuedOrder_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id =
               Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'",
               id);
                try
                {
                    var operationResult = _orderLogic.DeliveryOrder(CreateBindingModel(id));
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    _logger.LogInformation("Заказ №{id} выдан", id);
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о выдачи заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                   MessageBoxIcon.Error);
                }
            }

        }

        private void buttonRef_Click(object sender, EventArgs e)
        {
            LoadData();
        }
        private OrderBindingModel CreateBindingModel(int id, bool isDone = false)
        {
            return new OrderBindingModel
            {
                Id = id,
                DocumentId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["DocumentId"].Value),
                Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
                Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value),
                Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
                DateCreate = DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
            };
        }

        private void списокПакетовДокументовToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _reportLogic.SaveDocumentsToWordFile(new ReportBindingModel
                {
                    FileName = dialog.FileName
                });
                MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK,
               MessageBoxIcon.Information);
            }
        }

        private void компонентыПоПакетамДокументовToolStripMenuItem_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormReportDocumentComponents>();
			form.ShowDialog();
            

        }

        private void списокЗаказовToolStripMenuItem_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormReportOrders>();
			form.ShowDialog();
            
        }

        private void клиентыToolStripMenuItem_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormClients>();
			form.ShowDialog();
            
        }

        private void исполнителиToolStripMenuItem_Click(object sender, EventArgs e)
        {
			var form = DependencyManager.Instance.Resolve<FormImplementers>();
			form.ShowDialog();
            
        }

        private void запускРаботToolStripMenuItem_Click(object sender, EventArgs e)
        {
			_workProcess.DoWork(DependencyManager.Instance.Resolve<IImplementerLogic>(), _orderLogic);
			MessageBox.Show("Процесс обработки запущен", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

		private void почтаToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var form = DependencyManager.Instance.Resolve<FormMail>();
			form.ShowDialog();
			
		}

		private void создатьБэкапToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				if (_backUpLogic != null)
				{
					var fbd = new FolderBrowserDialog();
					if (fbd.ShowDialog() == DialogResult.OK)
					{
						_backUpLogic.CreateBackUp(new BackUpSaveBinidngModel
						{
							FolderName = fbd.SelectedPath
						});
						MessageBox.Show("Backup created", "Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
					}
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
	}
}
